# -*- coding: utf-8 -*-

from AccessControl import Unauthorized
from collective import iconifiedcategory as collective_iconifiedcategory
from collective.documentviewer.settings import GlobalSettings
from collective.iconifiedcategory.tests.base import BaseTestCase
from collective.iconifiedcategory.utils import get_category_object
from plone import api
from Products.Five import zcml


class TestCategorizedChildView(BaseTestCase):

    def setUp(self):
        super(TestCategorizedChildView, self).setUp()
        api.content.create(
            id='docB',
            type='Document',
            title='B',
            container=self.portal,
            content_category='config_-_group-1_-_category-1-2',
            to_print=False,
            confidential=False,
        )
        api.content.create(
            id='docA',
            type='Document',
            title='A',
            container=self.portal,
            content_category='config_-_group-1_-_category-1-2',
            to_print=False,
            confidential=False,
        )
        self.view = self.portal.restrictedTraverse('@@categorized-childs')
        self.view.portal_type = None

    def tearDown(self):
        super(TestCategorizedChildView, self).tearDown()
        elements = ('docB', 'docA')
        for element in elements:
            if element in self.portal:
                api.content.delete(self.portal[element])

    def test__call__(self):
        category = get_category_object(self.portal.file_txt,
                                       self.portal.file_txt.content_category)
        scale = category.restrictedTraverse('@@images').scale(scale='listing').__name__
        # the category and elements of category is displayed
        result = self.view()
        self.assertTrue(
            u'<img width="16px" height="16px" src="{0}/@@images/{1}"'.format(
                category.absolute_url(), scale) in result)

        # remove the categorized elements
        api.content.delete(self.portal['file_txt'])
        api.content.delete(self.portal['image'])
        api.content.delete(self.portal['docB'])
        api.content.delete(self.portal['docA'])
        self.assertEqual(self.view().strip(), u'<span class="discreet">Nothing.</span>')

    def test_categories_infos(self):
        self.view()
        infos = self.view.categories_infos()
        self.assertEqual(2, len(infos))
        self.assertEqual('category-1-1', infos[1]['id'])
        self.assertEqual(2, infos[0]['counts'])


class TestCategorizedChildInfosView(TestCategorizedChildView):

    def setUp(self):
        super(TestCategorizedChildInfosView, self).setUp()
        self.viewinfos = self.portal.restrictedTraverse('@@categorized-childs-infos')
        category_uid = self.config['group-1']['category-1-1'].UID()
        self.viewinfos(category_uid, filters={})

    def test__call__(self):
        # the category and elements of category is displayed
        self.viewinfos.update()
        result = self.viewinfos.index()
        self.assertTrue('<a class="categorized-element-title" href="http://nohost/plone/image/@@download">' in result)
        self.assertTrue('<span title="File description">file.txt</span>' in result)
        self.assertTrue(u'<a class="categorized-element-title" href="http://nohost/plone/image/@@download">' in result)
        self.assertTrue(u'<span title="Image description">ic\xf4ne1.png</span>' in result)

        # in case a file is too large, a warning is displayed
        # manipulate stored categorized_elements
        self.portal.categorized_elements[self.portal['file_txt'].UID()]['warn_filesize'] = True
        self.portal.categorized_elements[self.portal['file_txt'].UID()]['filesize'] = 7000000
        self.viewinfos.update()
        self.assertTrue("(<span class=\'warn_filesize\' title=\'Annex size is huge, "
                        "it could be difficult to be downloaded!\'>6.7 MB</span>)" in self.viewinfos.index())

        # remove the categorized elements
        api.content.delete(self.portal['file_txt'])
        api.content.delete(self.portal['image'])
        api.content.delete(self.portal['docB'])
        api.content.delete(self.portal['docA'])
        self.viewinfos.update()
        self.assertEqual(self.viewinfos.index(), u'\n')

    def test_categories_uids(self):
        self.viewinfos.update()
        self.assertEqual(
            [self.viewinfos.category_uid],
            self.viewinfos.categories_uids,
        )
        self.viewinfos.category_uid = self.config['group-1']['category-1-2'].UID()
        self.viewinfos.update()
        self.assertEqual(
            [self.viewinfos.category_uid],
            self.viewinfos.categories_uids,
        )

    def test_infos(self):
        self.viewinfos.update()
        infos = self.viewinfos.infos()
        self.assertItemsEqual([self.viewinfos.category_uid], infos.keys())
        self.assertItemsEqual(
            ['file.txt', 'ic\xc3\xb4ne1.png'],
            [e['title'] for e in infos[self.viewinfos.category_uid]],
        )

        self.viewinfos.category_uid = self.config['group-1']['category-1-2'].UID()
        self.viewinfos.update()
        infos = self.viewinfos.infos()
        self.assertItemsEqual([self.viewinfos.category_uid], infos.keys())
        self.assertItemsEqual(
            ['A', 'B'],
            [e['title'] for e in infos[self.viewinfos.category_uid]],
        )

    def test_filters(self):
        self.viewinfos.update()
        self.assertEqual(len(self.viewinfos.categorized_elements), 2)
        self.viewinfos.filters['id'] = 'file_txt'
        self.viewinfos.update()
        self.assertEqual(len(self.viewinfos.categorized_elements), 1)
        self.assertEqual(self.viewinfos.categorized_elements[0]['id'], 'file_txt')
        # filters are passed to viewinfos as json
        self.viewinfos(category_uid=self.viewinfos.category_uid,
                       filters={"id": "image"})
        self.assertEqual(len(self.viewinfos.categorized_elements), 1)
        self.assertEqual(self.viewinfos.categorized_elements[0]['id'], 'image')

    def test_show_preview(self):
        infos = self.portal.restrictedTraverse('@@categorized-childs-infos')
        gsettings = GlobalSettings(self.portal)
        gsettings.auto_convert = False
        gsettings.auto_layout_file_types = ['pdf']
        category_group = self.portal.config.get('group-1')
        category = category_group.get('category-1-1')
        category_uid = category.UID()
        file1 = api.content.create(
            id='file1',
            type='File',
            file=self.file_pdf,
            container=self.portal,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        # show_preview=0, default element was not converted
        element = self.portal.categorized_elements[file1.UID()]
        self.assertEqual(element['show_preview'], 0)
        self.assertEqual(element['preview_status'], 'not_converted')
        self.assertFalse('/file1/documentviewer#document/p1' in infos(category_uid, {}))
        self.assertTrue('/file1/@@download' in infos(category_uid, {}))
        # show_preview=1, element is converted and download is still possible
        category.show_preview = 1
        file2 = api.content.create(
            id='file2',
            type='File',
            file=self.file_pdf,
            container=self.portal,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        element = self.portal.categorized_elements[file2.UID()]
        self.assertEqual(element['show_preview'], 1)
        self.assertEqual(element['preview_status'], 'converted')
        self.assertTrue('/file2/documentviewer#document/p1' in infos(category_uid, {}))
        self.assertTrue('/file2/@@download' in infos(category_uid, {}))
        # show_preview=2, element is converted and download can be protected
        category.show_preview = 2
        file3 = api.content.create(
            id='file3',
            type='File',
            file=self.file_pdf,
            container=self.portal,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        element = self.portal.categorized_elements[file3.UID()]
        self.assertEqual(element['show_preview'], 2)
        self.assertEqual(element['preview_status'], 'converted')
        self.assertTrue('/file3/documentviewer#document/p1' in infos(category_uid, {}))
        self.assertTrue('/file3/@@download' in infos(category_uid, {}))


class TestCanViewAwareDownload(BaseTestCase):

    def test_default(self):
        # by default @@download returns the file, here
        # it is also the case as IIconifiedContent.can_view adapter returns True by default
        file_obj = self.portal['file_txt']
        img_obj = self.portal['image']
        self.assertTrue(file_obj.restrictedTraverse('@@download')())
        self.assertTrue(file_obj.restrictedTraverse('@@display-file')())
        self.assertTrue(file_obj.unrestrictedTraverse('view/++widget++form.widgets.file/@@download')())
        self.assertTrue(img_obj.restrictedTraverse('@@download')())
        self.assertTrue(img_obj.restrictedTraverse('@@display-file')())
        self.assertTrue(img_obj.unrestrictedTraverse('view/++widget++form.widgets.image/@@download')())

    def test_can_not_view(self):
        # register an adapter that will return False
        zcml.load_config('testing-adapters.zcml', collective_iconifiedcategory)
        file_obj = self.portal['file_txt']
        img_obj = self.portal['image']
        # downloadable when element is not confidential
        self.assertFalse(file_obj.confidential)
        self.assertFalse(img_obj.confidential)
        self.assertTrue(file_obj.restrictedTraverse('@@download')())
        self.assertTrue(file_obj.restrictedTraverse('@@display-file')())
        self.assertTrue(file_obj.unrestrictedTraverse('view/++widget++form.widgets.file/@@download')())
        self.assertTrue(img_obj.restrictedTraverse('@@download')())
        self.assertTrue(img_obj.restrictedTraverse('@@display-file')())
        self.assertTrue(img_obj.unrestrictedTraverse('view/++widget++form.widgets.image/@@download')())
        # when confidential, check can_view is done
        file_obj.confidential = True
        img_obj.confidential = True
        self.assertRaises(Unauthorized, file_obj.restrictedTraverse('@@download'))
        self.assertRaises(Unauthorized, file_obj.restrictedTraverse('@@display-file'))
        self.assertRaises(Unauthorized, file_obj.unrestrictedTraverse('view/++widget++form.widgets.file/@@download'))
        self.assertRaises(Unauthorized, img_obj.restrictedTraverse('@@download'))
        self.assertRaises(Unauthorized, img_obj.restrictedTraverse('@@display-file'))
        self.assertRaises(Unauthorized, img_obj.unrestrictedTraverse('view/++widget++form.widgets.image/@@download'))
        # when using show_preview == 2, download is disabled
        # ths widget is shown but downloading raises Unauthorized
        file_obj.confidential = False
        img_obj.confidential = False
        self.assertTrue(file_obj.restrictedTraverse('@@download')())
        self.assertTrue(img_obj.restrictedTraverse('@@download')())
        self.portal.categorized_elements[file_obj.UID()]['show_preview'] = 2
        self.portal.categorized_elements[img_obj.UID()]['show_preview'] = 2
        self.assertRaises(Unauthorized, file_obj.restrictedTraverse('@@download'))
        self.assertRaises(Unauthorized, file_obj.restrictedTraverse('@@display-file'))
        self.assertTrue(file_obj.unrestrictedTraverse('view/++widget++form.widgets.file/@@download')())
        self.assertRaises(Unauthorized, img_obj.restrictedTraverse('@@download'))
        self.assertRaises(Unauthorized, img_obj.restrictedTraverse('@@display-file'))
        self.assertTrue(img_obj.unrestrictedTraverse('view/++widget++form.widgets.image/@@download')())
        # cleanUp zmcl.load_config because it impacts other tests
        zcml.cleanUp()



# -*- coding: utf-8 -*-
"""Installer for the collective.iconifiedcategory package."""

from setuptools import find_packages
from setuptools import setup


long_description = (
    open('README.rst').read() +
    '\n' +
    'Contributors\n' +
    '============\n' +
    '\n' +
    open('CONTRIBUTORS.rst').read() +
    '\n' +
    open('CHANGES.rst').read() +
    '\n')


setup(
    name='collective.iconifiedcategory',
    version='0.61',
    description="An add-on for Plone",
    long_description=long_description,
    # Get more from https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 6 - Mature",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords='Python Plone',
    author='IMIO',
    author_email='support@imio.be',
    url='https://pypi.python.org/pypi/collective.iconifiedcategory',
    license='GPL version 2',
    packages=find_packages('src', exclude=['ez_setup']),
    namespace_packages=['collective'],
    package_dir={'': 'src'},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'collective.documentviewer<5',
        'collective.eeafaceted.z3ctable>2.17',
        'collective.fontawesome>=1.1',
        'collective.js.tooltipster>0.1',
        'collective.z3cform.select2',
        'imio.helpers',
        'natsort',
        'plone.api>=1.4.11',
        'plone.app.contenttypes',
        'plone.app.dexterity',
        'plone.autoform',
        'plone.formwidget.namedfile',
        'plone.namedfile',
        'setuptools',
        'z3c.jbot',
        'z3c.json',
        'z3c.table',
    ],
    extras_require={
        'test': [
            'plone.app.testing',
            'plone.app.contenttypes',
            'plone.app.robotframework[debug]',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Acquisition import aq_base
from collective.iconifiedcategory.content.base import ICategorize
from collective.iconifiedcategory.content.categoryconfiguration import ICategoryConfiguration
from collective.iconifiedcategory.content.categorygroup import ICategoryGroup
from collective.iconifiedcategory.interfaces import IIconifiedCategorySubtyper
from Products.Five import BrowserView
from zope.interface import implements


class IconifiedCategoryPublicSubtyper(BrowserView):
    implements(IIconifiedCategorySubtyper)

    def __init__(self, context, request):
        self.context = aq_base(context)
        self.request = request

    @property
    def have_categorized_elements(self):
        """See IIconifiedCategorySubtyper"""
        return False

    @property
    def on_config(self):
        """Verifiy if we are on a config or category folder"""
        interfaces = (
            ICategorize,
            ICategoryConfiguration,
            ICategoryGroup,
        )
        for interface in interfaces:
            if interface.providedBy(self.context):
                return True
        return False


class IconifiedCategorySubtyper(IconifiedCategoryPublicSubtyper):

    @property
    def have_categorized_elements(self):
        """See IIconifiedCategorySubtyper"""
        return len(getattr(self.context, 'categorized_elements', {})) > 0



# -*- coding: utf-8 -*-

from z3c.form import interfaces
from z3c.form.browser.select import SelectWidget
from z3c.form.widget import FieldWidget
from zope.component import adapter
from zope.interface import implementer
from zope.interface import implements
from zope.interface import Interface
from zope.schema.interfaces import IChoice


class ICategoryTitleWidget(Interface):
    """Marker interface for hidden select widget"""


class CategoryTitleWidget(SelectWidget):
    implements(ICategoryTitleWidget, interfaces.ISelectWidget)

    @property
    def placeholder(self):
        return self.field.placeholder

    @property
    def select2_id(self):
        return self.id.replace('-', '_')


@adapter(IChoice, interfaces.IFormLayer)
@implementer(interfaces.IFieldWidget)
def CategoryTitleFieldWidget(field, request):
    """IFieldWidget factory for CategoryTitleWidget"""
    return FieldWidget(field, CategoryTitleWidget(request))



# -*- coding: utf-8 -*-

from AccessControl import Unauthorized
from Acquisition import aq_base
from Acquisition import aq_inner
from collections import OrderedDict
from collective.iconifiedcategory.interfaces import IIconifiedCategorySettings
from collective.iconifiedcategory.interfaces import IIconifiedContent
from collective.iconifiedcategory.utils import boolean_message
from collective.iconifiedcategory.utils import get_categorized_elements
from collective.iconifiedcategory.utils import print_message
from collective.iconifiedcategory.utils import render_filesize
from collective.iconifiedcategory.utils import signed_message
from DateTime import DateTime
from plone import api
from plone.formwidget.namedfile.widget import Download as fnw_Download
from plone.namedfile.browser import DisplayFile
from plone.namedfile.browser import Download
from plone.namedfile.scaling import ImageScaling
from plone.rfc822.interfaces import IPrimaryFieldInfo
from Products.CMFCore.permissions import View
from Products.CMFCore.utils import _checkPermission
from Products.Five import BrowserView
from zope.component import getMultiAdapter
from zope.component.interfaces import ComponentLookupError

import json


class CategorizedChildView(BrowserView):
    """ """

    @property
    def _filters(self):
        """Overridable method to define custom filters."""
        return {}

    def _filters_json(self):
        """Filters are stored in template as json."""
        return json.dumps(self._filters)

    def update(self):
        self.categorized_elements = get_categorized_elements(
            self.context,
            portal_type=self.portal_type,
            filters=self._filters,
            check_can_view=self.check_can_view,
        )

    def __call__(self, portal_type=None, show_nothing=True, check_can_view=False):
        """We set p_check_can_view=False by default."""
        self.portal_url = api.portal.get().absolute_url()
        self.portal_type = portal_type
        self.show_nothing = show_nothing
        self.check_can_view = check_can_view
        self.update()
        return super(CategorizedChildView, self).__call__()

    def has_elements_to_show(self):
        return len(self.categorized_elements)

    def categories_infos(self):
        infos = [(e['category_uid'], {'id': e['category_id'],
                                      'uid': e['category_uid'],
                                      'title': e['category_title'],
                                      'counts': 0,
                                      'icon': e['icon_url']})
                 for e in self.categorized_elements]
        infos = OrderedDict(infos)
        for key, element in infos.items():
            element['counts'] = len([e for e in self.categorized_elements
                                     if e['category_uid'] == key])
        return infos.values()


class ManageCategorizedChildView(BrowserView):
    """ """

    def get_management_url(self):
        """Overridable method to define management url."""
        return "{0}/@@iconifiedcategory".format(self.context.absolute_url())


class CategorizedChildInfosView(BrowserView):
    """ """
    def __init__(self, context, request):
        """ """
        super(CategorizedChildInfosView, self).__init__(context, request)
        self.portal_url = api.portal.get().absolute_url()

    def update(self):
        self.filters['category_uid'] = self.category_uid
        self.categorized_elements = get_categorized_elements(
            self.context,
            filters=self.filters)
        self._infos = self.infos()

    def __call__(self, category_uid, filters):
        """ """
        self.category_uid = category_uid
        self.filters = filters
        self.update()
        return super(CategorizedChildInfosView, self).__call__()

    def show_details(self, number_of_columns):
        """Only show details if displaying at most 2 columns of elements
           in the tooltispter popup."""
        return bool(number_of_columns < 3)

    def show_preview(self, element):
        """Made to be overrided."""
        return element["show_preview"] and \
            element["preview_status"] in ('in_progress', 'converted')

    @property
    def categories_uids(self):
        return OrderedDict.fromkeys(
            [e['category_uid'] for e in self.categorized_elements],
        ).keys()

    def infos(self):
        infos = OrderedDict([(e, []) for e in self.categories_uids])
        for element in self.categorized_elements:
            infos[element['category_uid']].append(element)
        self._infos = infos
        return infos

    def render_filesize(self, size):
        """ """
        return render_filesize(size)

    def show_more_infos_url(self):
        """
        In some case we show categorized content but user does not have
        access to the context, in the case do not show the more infos link.
        """
        if _checkPermission(View, self.context):
            return True

    def categorized_elements_more_infos_url(self):
        """ """
        return "{0}/{1}".format(self.context.absolute_url(), "@@iconifiedcategory")

    def number_of_columns(self, elements):
        """Return number of columns to display categorized_elements on
           when displaying many elements."""
        columns_treshold = api.portal.get_registry_record(
            'categorized_childs_infos_columns_threshold',
            interface=IIconifiedCategorySettings,
        )
        columns_treshold = float(columns_treshold)
        return round(len(elements) / columns_treshold)

    def show(self, element, attr_prefix):
        """ """
        show = element['{0}_activated'.format(attr_prefix)] and self._show_detail(attr_prefix)
        return show

    def show_download(self, element):
        """ """
        return not element["show_preview"] == 2 or \
            (element["show_preview"] == 2 and self._show_protected_download(element))

    def _show_protected_download(self, element):
        """When "show_preview" is "2", trigger advanced check.
           Made to be overrided."""
        return True

    def _show_detail(self, detail_type):
        """Made to be overrided."""
        return True

    def get_css_classses_for(self, functionnality, element):
        """ """
        css_classes = []
        if functionnality == "to_print":
            css_classes.append("iconified-print")
            if element['to_print'] is None:
                css_classes.append('deactivated')
            elif element['to_print'] is True:
                css_classes.append('active')
        elif functionnality == "signed":
            css_classes.append("iconified-signed")
            if element['to_sign'] is False:
                css_classes.append('deactivated')
            elif element['signed'] is True:
                css_classes.append('active')
        else:
            # default behavior
            css_classes.append("iconified-{0}".format(functionnality))
            if element[functionnality] is True:
                css_classes.append('active')
        return " ".join(css_classes)

    def get_tag_title_for(self, functionnality, element):
        """ """
        msg = ''
        if functionnality == "to_print":
            msg = print_message(to_print_value=element['to_print'])
        elif functionnality == "signed":
            msg = signed_message(to_sign_value=element['to_sign'],
                                 signed_value=element['signed'])
        else:
            # default behavior, a boolean message
            msg = boolean_message(attr_name=functionnality,
                                  value=element[functionnality])
        return msg


def check_can_view(obj, request):
    """ """
    try:
        adapter = getMultiAdapter((obj.aq_parent, request, obj),
                                  IIconifiedContent)
        return adapter.can_view()
    except ComponentLookupError:
        return True


class CanViewAwareDownload(Download):
    """ """
    def __call__(self):
        if not check_can_view(self.context, self.request):
            raise Unauthorized
        else:
            # when using preview, check if downloadable
            parent = self.context.aq_parent
            element = parent.categorized_elements[self.context.UID()]
            if element['show_preview'] != 0:
                infos = parent.unrestrictedTraverse('@@categorized-childs-infos')
                if not infos.show_download(element):
                    raise Unauthorized
        # access is managed by can_view
        with api.env.adopt_roles(['Manager']):
            return super(CanViewAwareDownload, self).__call__()


class CanViewAwareDisplayFile(DisplayFile, CanViewAwareDownload):
    """ """


class CanViewAwareFNWDownload(fnw_Download):
    """ """
    def __call__(self):
        if not check_can_view(aq_inner(self.context.context), self.request):
            raise Unauthorized
        # access is managed by can_view
        with api.env.adopt_roles(['Manager']):
            return super(CanViewAwareFNWDownload, self).__call__()


class ImageDataModifiedImageScaling(ImageScaling):
    """ """

    def modified(self):
        """Returns the stored file _p_mtime instead content _p_mtime."""
        context = aq_base(self.context)
        value = IPrimaryFieldInfo(context).value
        date = DateTime(value._p_mtime)
        return date.millis()



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.documentviewer.async import queueJob
from collective.documentviewer.config import CONVERTABLE_TYPES
from collective.documentviewer.settings import GlobalSettings
from collective.iconifiedcategory import adapter
from collective.iconifiedcategory.content.subcategory import ISubcategory
from collective.iconifiedcategory.interfaces import IIconifiedContent
from collective.iconifiedcategory.tests.base import BaseTestCase
from collective.iconifiedcategory.utils import get_category_object
from plone.app.contenttypes.interfaces import IFile
from plone.app.contenttypes.interfaces import IImage
from plone.app.contenttypes.interfaces import ILink
from plone.namedfile.utils import stream_data
from time import sleep
from zope.annotation import IAnnotations
from zope.component import getMultiAdapter
from zope.event import notify
from zope.interface import alsoProvides
from zope.lifecycleevent import ObjectModifiedEvent


class TestCategorizedObjectInfoAdapter(BaseTestCase):

    def test_get_infos_for_file(self):
        obj = self.portal['file_txt']
        file_adapter = adapter.CategorizedObjectInfoAdapter(
            obj)
        category = get_category_object(obj, obj.content_category)
        infos = file_adapter.get_infos(category)
        scale = category.restrictedTraverse('@@images').scale(scale='listing').__name__
        self.assertEqual(
            infos,
            {'allowedRolesAndUsers': ['Anonymous'],
             'category_id': category.category_id,
             'category_title': category.category_title,
             'category_uid': category.category_uid,
             'confidential': False,
             'confidentiality_activated': False,
             'contentType': 'text/plain',
             'description': obj.Description(),
             'download_url': u'file_txt/@@download',
             'filesize': 3017,
             'icon_url': u'config/group-1/category-1-1/@@images/{0}'.format(scale),
             'id': obj.getId(),
             'last_updated': self._modified(obj),
             'portal_type': obj.portal_type,
             'preview_status': 'not_convertable',
             'publishable': False,
             'publishable_activated': False,
             'relative_url': 'file_txt',
             'show_preview': 0,
             'signed': False,
             'signed_activated': False,
             'subcategory_id': None,
             'subcategory_title': None,
             'subcategory_uid': None,
             'title': obj.Title(),
             'to_be_printed_activated': True,
             'to_print': None,
             'to_sign': False,
             'warn_filesize': False})

    def test_get_infos_for_image(self):
        obj = self.portal['image']
        image_adapter = adapter.CategorizedObjectInfoAdapter(
            obj)
        # image use a subcategory
        subcategory = get_category_object(obj, obj.content_category)
        infos = image_adapter.get_infos(subcategory)
        category = subcategory.get_category()
        scale = category.restrictedTraverse('@@images').scale(scale='listing').__name__
        self.assertEqual(
            infos,
            {'allowedRolesAndUsers': ['Anonymous'],
             'category_id': subcategory.category_id,
             'category_title': subcategory.category_title,
             'category_uid': subcategory.category_uid,
             'confidential': False,
             'confidentiality_activated': False,
             'contentType': 'image/png',
             'description': obj.Description(),
             'download_url': u'image/@@download',
             'filesize': 3742,
             'icon_url': u'config/group-1/category-1-1/@@images/{0}'.format(scale),
             'id': obj.getId(),
             'last_updated': self._modified(obj),
             'portal_type': obj.portal_type,
             'preview_status': 'not_convertable',
             'publishable': False,
             'publishable_activated': False,
             'relative_url': 'image',
             'show_preview': 0,
             'signed': False,
             'signed_activated': False,
             'subcategory_id': subcategory.id,
             'subcategory_title': subcategory.Title(),
             'subcategory_uid': subcategory.UID(),
             'title': obj.Title(),
             'to_be_printed_activated': True,
             'to_print': False,
             'to_sign': False,
             'warn_filesize': False})

    def test_get_infos_with_subcategory(self):
        obj = self.portal['file_txt']
        file_adapter = adapter.CategorizedObjectInfoAdapter(
            obj)
        subcategory = self.portal.restrictedTraverse('config/group-1/category-1-1/subcategory-1-1-1')
        self.assertTrue(ISubcategory.providedBy(subcategory))
        infos = file_adapter.get_infos(subcategory)
        category = subcategory.get_category()
        scale = category.restrictedTraverse('@@images').scale(scale='listing').__name__
        self.assertEqual(
            infos,
            {'allowedRolesAndUsers': ['Anonymous'],
             'category_id': subcategory.category_id,
             'category_title': subcategory.category_title,
             'category_uid': subcategory.category_uid,
             'confidential': False,
             'confidentiality_activated': False,
             'contentType': 'text/plain',
             'description': obj.Description(),
             'download_url': u'file_txt/@@download',
             'filesize': 3017,
             'icon_url': u'config/group-1/category-1-1/@@images/{0}'.format(scale),
             'id': obj.getId(),
             'last_updated': self._modified(obj),
             'portal_type': obj.portal_type,
             'preview_status': 'not_convertable',
             'publishable': False,
             'publishable_activated': False,
             'relative_url': 'file_txt',
             'show_preview': 0,
             'signed': False,
             'signed_activated': False,
             'subcategory_id': subcategory.getId(),
             'subcategory_title': subcategory.Title(),
             'subcategory_uid': subcategory.UID(),
             'title': obj.Title(),
             'to_be_printed_activated': True,
             'to_print': None,
             'to_sign': False,
             'warn_filesize': False})

    def test_category(self):
        file_adapter = adapter.CategorizedObjectInfoAdapter(
            self.portal['file_txt'])
        self.assertEqual('config_-_group-1_-_category-1-1',
                         file_adapter._category)

        image_adapter = adapter.CategorizedObjectInfoAdapter(
            self.portal['image'])
        self.assertEqual('config_-_group-1_-_category-1-1',
                         image_adapter._category)

    def test_filesize(self):
        image_adapter = adapter.CategorizedObjectInfoAdapter(
            self.portal['image'])
        self.assertEqual(3742, image_adapter._filesize)

        file_adapter = adapter.CategorizedObjectInfoAdapter(
            self.portal['file_txt'])
        self.assertEqual(3017, file_adapter._filesize)

    def test_download_url(self):
        image = self.portal['image']
        image_adapter = adapter.CategorizedObjectInfoAdapter(image)
        image_download_url = image_adapter._download_url
        self.assertEqual(image_download_url, u'image/@@download')
        # element is really downloadable
        download_view = self.portal.restrictedTraverse(str(image_download_url))
        self.assertEqual(stream_data(image.image), download_view())

        file_obj = self.portal['file_txt']
        file_adapter = adapter.CategorizedObjectInfoAdapter(file_obj)
        file_download_url = file_adapter._download_url
        self.assertEqual(file_download_url, u'file_txt/@@download')
        # element is really downloadable
        download_view = self.portal.restrictedTraverse(str(file_download_url))
        self.assertEqual(stream_data(file_obj.file), download_view())

    def test_preview_status(self):
        image_adapter = adapter.CategorizedObjectInfoAdapter(
            self.portal['image'])
        self.assertEqual('not_convertable',
                         image_adapter._preview_status)

        file_adapter = adapter.CategorizedObjectInfoAdapter(
            self.portal['file_txt'])
        self.assertEqual('not_convertable',
                         file_adapter._preview_status)

    def test_uri_are_relative_to_portal(self):
        obj = self.portal['file_txt']
        file_adapter = adapter.CategorizedObjectInfoAdapter(
            obj)
        category = get_category_object(obj, obj.content_category)
        infos = file_adapter.get_infos(category)
        portal_url = self.portal.absolute_url()
        self.assertTrue(not infos['relative_url'].startswith(portal_url))
        self.assertTrue(not infos['download_url'].startswith(portal_url))
        self.assertTrue(not infos['icon_url'].startswith(portal_url))


class TestCategorizedObjectAdapter(BaseTestCase):

    def test_can_view(self):
        cat_adapter = getMultiAdapter((self.portal, self.portal.REQUEST, self.portal['file_txt']),
                                      IIconifiedContent)
        self.assertTrue(cat_adapter.can_view())


class TestCategorizedObjectPrintableAdapter(BaseTestCase):

    def setUp(self):
        super(TestCategorizedObjectPrintableAdapter, self).setUp()
        gsettings = GlobalSettings(self.portal)
        gsettings.auto_layout_file_types = CONVERTABLE_TYPES.keys()

    def test_is_printable_default(self):
        obj = self.portal.file_txt
        print_adapter = adapter.CategorizedObjectPrintableAdapter(obj)
        self.assertTrue(print_adapter.is_printable)

    def test_is_printable_link(self):
        obj = self.portal.file_txt
        alsoProvides(obj, ILink)
        print_adapter = adapter.CategorizedObjectPrintableAdapter(obj)
        self.assertFalse(print_adapter.is_printable)

    def test_is_printable_image(self):
        obj = self.portal.file_txt
        alsoProvides(obj, IImage)
        print_adapter = adapter.CategorizedObjectPrintableAdapter(obj)
        self.assertTrue(print_adapter.is_printable)

    def test_is_printable_file(self):
        obj = self.portal.file_txt
        alsoProvides(obj, IFile)
        print_adapter = adapter.CategorizedObjectPrintableAdapter(obj)
        self.assertTrue(print_adapter.is_printable)

    def test_update_object(self):
        obj = self.portal.file_txt
        alsoProvides(obj, IFile)
        # will be converted with collective.documentviewer
        obj.to_print = True
        notify(ObjectModifiedEvent(obj))
        self.assertIsNone(obj.to_print_message)
        self.assertTrue(obj.to_print)

        obj.file.contentType = 'audio/mpeg3'
        obj.to_print = True
        notify(ObjectModifiedEvent(obj))
        self.assertEqual(u'Can not be printed', obj.to_print_message)
        self.assertFalse(obj.to_print)


class TestCategorizedObjectPreviewAdapter(BaseTestCase):

    def test_is_convertible(self):
        obj = self.portal['file_txt']

        preview_adapter = adapter.CategorizedObjectPreviewAdapter(obj)

        # convertible relies on the fact that contentType is managed
        # by collective.documentviewer gsettings.auto_layout_file_types
        gsettings = GlobalSettings(self.portal)
        self.assertEqual(gsettings.auto_layout_file_types, ['pdf'])

        obj.file.contentType = 'application/pdf'
        self.assertTrue(preview_adapter.is_convertible())

        obj.file.contentType = 'application/rtf'
        self.assertFalse(preview_adapter.is_convertible())

        # right enable every file_types in collective.documentviewer
        gsettings.auto_layout_file_types = CONVERTABLE_TYPES.keys()

        convertables = (
            'application/msword',
            'application/pdf',
            'application/rtf',
            'application/vnd.oasis.opendocument.spreadsheet',
            'application/vnd.oasis.opendocument.text',
            # xlsx
            'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
            'image/png',
            'image/jpeg',
            'text/html',
        )
        for convertable in convertables:
            obj.file.contentType = convertable
            self.assertTrue(preview_adapter.is_convertible())

        not_convertables = ('application/octet-stream',
                            'text/x-python')
        for not_convertable in not_convertables:
            obj.file.contentType = not_convertable
            self.assertFalse(preview_adapter.is_convertible())

    def test_status(self):
        obj = self.portal['file_txt']

        preview_adapter = adapter.CategorizedObjectPreviewAdapter(obj)

        gsettings = GlobalSettings(self.portal)
        self.assertEqual(gsettings.auto_layout_file_types, ['pdf'])

        obj.file.contentType = 'application/rtf'
        self.assertEqual(preview_adapter.status, 'not_convertable')
        self.assertFalse(preview_adapter.converted)

        obj.file.contentType = 'application/pdf'
        self.assertEqual(preview_adapter.status, 'not_converted')
        self.assertFalse(preview_adapter.converted)

        ann = IAnnotations(obj)['collective.documentviewer']
        ann['filehash'] = '--foobar--'
        obj.file.contentType = 'application/pdf'
        self.assertEqual(preview_adapter.status, 'in_progress')
        self.assertFalse(preview_adapter.converted)

        queueJob(obj)
        # not a real PDF actually...
        self.assertEqual(preview_adapter.status, 'conversion_error')
        self.assertFalse(preview_adapter.converted)

        # enable every supported types including txt
        gsettings.auto_layout_file_types = CONVERTABLE_TYPES.keys()
        obj.file.contentType = 'text/plain'
        # collective.documentviewer checks if element was modified
        # or it does not convert again
        sleep(1)
        obj.notifyModified()
        queueJob(obj)
        self.assertEqual(preview_adapter.status, 'converted')
        self.assertTrue(preview_adapter.converted)



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory.content.base import ICategorize
from plone.app.contenttypes.interfaces import IFolder
from plone.dexterity.content import Item
from plone.dexterity.schema import DexteritySchemaPolicy
from zope.interface import implements


class ISubcategory(IFolder, ICategorize):
    pass


class Subcategory(Item):
    implements(ISubcategory)

    @property
    def category_uid(self):
        return self.aq_parent.UID()

    @property
    def category_id(self):
        return self.aq_parent.getId()

    @property
    def category_title(self):
        return self.aq_parent.Title()

    def get_category(self):
        return self.aq_parent

    def get_category_group(self, context=None):
        if context is None:
            context = self
        return context.aq_parent.aq_parent


class SubcategorySchemaPolicy(DexteritySchemaPolicy):

    def bases(self, schema_name, tree):
        return (ISubcategory, )



# -*- coding: utf-8 -*-

from AccessControl import Unauthorized
from collective.documentviewer.config import CONVERTABLE_TYPES
from collective.documentviewer.settings import GlobalSettings
from collective.iconifiedcategory import utils
from collective.iconifiedcategory.browser.actionview import BaseView
from collective.iconifiedcategory.tests.base import BaseTestCase
from plone import api
from Products.CMFCore.permissions import ModifyPortalContent
from z3c.json.interfaces import IJSONReader
from zope.component import getUtility


class TestBaseView(BaseTestCase):

    def test__call__(self):
        obj = self.portal['file_txt']
        view = BaseView(obj, self.portal.REQUEST)
        reader = getUtility(IJSONReader)

        # when attribute_mapping is not set, it does not work
        result = reader.read(view())
        self.assertEqual(result[u'status'], 2)
        self.assertEqual(result[u'msg'], u'No values to set')

        # only doable if user has Modify portal content on obj
        view.attribute_mapping = {'title': 'action-value-title'}
        self.portal.REQUEST.set('action-value-title', 'My new title')
        obj.manage_permission(ModifyPortalContent, roles=[])
        result = reader.read(view())
        self.assertEqual(result[u'status'], 2)
        self.assertEqual(result[u'msg'], u'Error during process')
        obj.manage_permission(ModifyPortalContent, roles=['Manager'])

        # change title
        self.assertEqual(obj.title, u'file.txt')
        result = reader.read(view())
        self.assertEqual(result[u'status'], -1)
        self.assertEqual(result[u'msg'], u'Values have been set')
        self.assertEqual(obj.title, 'My new title')

    def test_get_current_values(self):
        obj = self.portal['file_txt']
        view = BaseView(obj, self.portal.REQUEST)

        self.assertEqual(view.get_current_values(), {})
        view.attribute_mapping = {'title': 'action-value-title'}
        self.assertEqual(view.get_current_values(), {'title': u'file.txt'})


class TestToPrintChangeView(BaseTestCase):

    def test_set_values(self):
        obj = self.portal['file_txt']
        view = obj.restrictedTraverse('@@iconified-print')

        # works only if functionnality enabled and user have Modify portal content
        category = utils.get_category_object(obj, obj.content_category)
        group = category.get_category_group()

        # fails if one of 2 conditions is not fullfilled
        self.assertTrue(api.user.has_permission(ModifyPortalContent, obj=obj))
        group.to_be_printed_activated = False
        self.assertRaises(Unauthorized, view.set_values, {'to_print': True})
        group.to_be_printed_activated = True

        obj.manage_permission(ModifyPortalContent, roles=[])
        self.assertRaises(Unauthorized, view.set_values, {'to_print': True})
        obj.manage_permission(ModifyPortalContent, roles=['Manager'])

        # set to None when format not managed by collective.documentviewer
        self.assertFalse(obj.to_print, obj.aq_parent.categorized_elements[obj.UID()]['to_print'])
        view.set_values({'to_print': True})
        self.assertIsNone(obj.to_print, obj.aq_parent.categorized_elements[obj.UID()]['to_print'])

        # will be correctly set if format is managed
        gsettings = GlobalSettings(self.portal)
        gsettings.auto_layout_file_types = CONVERTABLE_TYPES.keys()
        view.set_values({'to_print': True})
        self.assertTrue(obj.to_print, obj.aq_parent.categorized_elements[obj.UID()]['to_print'])
        view.set_values({'to_print': False})
        self.assertFalse(obj.to_print, obj.aq_parent.categorized_elements[obj.UID()]['to_print'])


class TestConfidentialChangeView(BaseTestCase):

    def test_set_values(self):
        obj = self.portal['file_txt']
        view = obj.restrictedTraverse('@@iconified-confidential')

        # works only if functionnality enabled and user have Modify portal content
        category = utils.get_category_object(obj, obj.content_category)
        group = category.get_category_group()

        # fails if one of 2 conditions is not fullfilled
        self.assertTrue(api.user.has_permission(ModifyPortalContent, obj=obj))
        group.confidentiality_activated = False
        self.assertRaises(Unauthorized, view.set_values, {'confidential': True})
        group.confidentiality_activated = True

        obj.manage_permission(ModifyPortalContent, roles=[])
        self.assertRaises(Unauthorized, view.set_values, {'confidential': True})
        obj.manage_permission(ModifyPortalContent, roles=['Manager'])

        # functionnality enabled and user have Modify portal content
        self.assertFalse(obj.confidential)
        view.set_values({'confidential': True})
        self.assertTrue(obj.confidential, obj.aq_parent.categorized_elements[obj.UID()]['confidential'])
        view.set_values({'confidential': False})
        self.assertFalse(obj.confidential, obj.aq_parent.categorized_elements[obj.UID()]['confidential'])


class TestSignedChangeView(BaseTestCase):

    def test_set_values(self):
        obj = self.portal['file_txt']
        view = obj.restrictedTraverse('@@iconified-signed')

        # works only if functionnality enabled and user have Modify portal content
        category = utils.get_category_object(obj, obj.content_category)
        group = category.get_category_group()

        # fails if one of 2 conditions is not fullfilled
        self.assertTrue(api.user.has_permission(ModifyPortalContent, obj=obj))
        group.signed_activated = False
        self.assertRaises(Unauthorized, view.set_values, {'to_sign': True})
        group.signed_activated = True

        obj.manage_permission(ModifyPortalContent, roles=[])
        self.assertRaises(Unauthorized, view.set_values, {'to_sign': True})
        obj.manage_permission(ModifyPortalContent, roles=['Manager'])

        # functionnality enabled and user have Modify portal content
        self.assertFalse(obj.to_sign, obj.aq_parent.categorized_elements[obj.UID()]['to_sign'])
        self.assertFalse(obj.signed, obj.aq_parent.categorized_elements[obj.UID()]['signed'])

        view.set_values({'to_sign': True})
        self.assertTrue(obj.to_sign, obj.aq_parent.categorized_elements[obj.UID()]['to_sign'])
        view.set_values({'signed': True})
        self.assertTrue(obj.signed, obj.aq_parent.categorized_elements[obj.UID()]['signed'])

        # multiple attributes may be set at the same time
        view.set_values({'to_sign': False, 'signed': False})
        self.assertFalse(obj.to_sign, obj.aq_parent.categorized_elements[obj.UID()]['to_sign'])
        self.assertFalse(obj.signed, obj.aq_parent.categorized_elements[obj.UID()]['signed'])
        view.set_values({'to_sign': True, 'signed': False})
        self.assertTrue(obj.to_sign, obj.aq_parent.categorized_elements[obj.UID()]['to_sign'])
        self.assertFalse(obj.signed, obj.aq_parent.categorized_elements[obj.UID()]['signed'])
        view.set_values({'to_sign': True, 'signed': True})
        self.assertTrue(obj.to_sign, obj.aq_parent.categorized_elements[obj.UID()]['to_sign'])
        self.assertTrue(obj.signed, obj.aq_parent.categorized_elements[obj.UID()]['signed'])

    def test_get_next_values(self):
        """to_sign/signed are logically linked and the action in the UI
           will loop among possible values :
           - to_sign False, signed False;
           - to_sign True, signed False,
           - to_sign True, signed True.
           to_sign False, signed True is not possible."""
        obj = self.portal['file_txt']
        view = obj.restrictedTraverse('@@iconified-signed')
        category = utils.get_category_object(obj, obj.content_category)
        group = category.get_category_group()
        group.signed_activated = True
        # loop between possibilities
        self.assertEqual(
            view._get_next_values({'to_sign': False, 'signed': False}),
            (0, {'to_sign': True, 'signed': False}))
        self.assertEqual(
            view._get_next_values({'to_sign': True, 'signed': False}),
            (1, {'to_sign': True, 'signed': True}))
        self.assertEqual(
            view._get_next_values({'to_sign': True, 'signed': True}),
            (-1, {'to_sign': False, 'signed': False}))
        # not possible values, back to 'to_sign'
        self.assertEqual(
            view._get_next_values({'to_sign': False, 'signed': True}),
            (0, {'to_sign': True, 'signed': False}))


class TestPublishableChangeView(BaseTestCase):

    def test_set_values(self):
        obj = self.portal['file_txt']
        view = obj.restrictedTraverse('@@iconified-publishable')

        # works only if functionnality enabled and user have Modify portal content
        category = utils.get_category_object(obj, obj.content_category)
        group = category.get_category_group()

        # fails if one of 2 conditions is not fullfilled
        self.assertTrue(api.user.has_permission(ModifyPortalContent, obj=obj))
        group.publishable_activated = False
        self.assertRaises(Unauthorized, view.set_values, {'publishable': True})
        group.publishable_activated = True

        obj.manage_permission(ModifyPortalContent, roles=[])
        self.assertRaises(Unauthorized, view.set_values, {'publishable': True})
        obj.manage_permission(ModifyPortalContent, roles=['Manager'])

        # functionnality enabled and user have Modify portal content
        self.assertFalse(obj.publishable)
        view.set_values({'publishable': True})
        self.assertTrue(obj.publishable, obj.aq_parent.categorized_elements[obj.UID()]['publishable'])
        view.set_values({'publishable': False})
        self.assertFalse(obj.publishable, obj.aq_parent.categorized_elements[obj.UID()]['publishable'])



# -*- coding: utf-8 -*-

from plone import api
from plone import namedfile

import os


def post_install(context):
    """Post install script"""
    if context.readDataFile('collectivecategorize_default.txt') is None:
        return


def post_test_install(context):
    if context.readDataFile('collectivecategorize_testing.txt') is None:
        return
    create_config(context)


def create_config(context):
    portal = api.portal.get()
    current_path = os.path.dirname(__file__)
    config = api.content.create(
        type='ContentCategoryConfiguration',
        title='Config',
        container=portal,
    )
    groups = []
    for idx in range(1, 3):
        obj = api.content.create(
            type='ContentCategoryGroup',
            title='Group {0}'.format(idx),
            container=config,
            to_be_printed_activated=True
        )
        groups.append(obj)
    for group_idx, group in enumerate(groups):
        for cat_idx in reversed(range(1, 4)):
            filename = u'icône{0}.png'.format(cat_idx)
            f = open(os.path.join(current_path, 'tests', filename), 'r')
            icon = namedfile.NamedBlobFile(f.read(), filename=filename)
            title = 'Category {0}-{1}'.format(group_idx + 1, cat_idx)
            category = api.content.create(
                type='ContentCategory',
                title=title,
                container=group,
                icon=icon,
                predefined_title=title,
            )
            for idx in reversed(range(1, 3)):
                api.content.create(
                    type='ContentSubcategory',
                    title='Subcategory {0}-{1}-{2}'.format(
                        group_idx + 1,
                        cat_idx,
                        idx,
                    ),
                    container=category,
                )
